( function( $, QUnit ) {

	"use strict";

	var $bodyContent = null,
			$fixture = null;

	QUnit.module( "jQuery Docout", {
		beforeEach: function() {
			var $content = $( "<div id=\"bodyContent\">" +
												"<table>" +
												"<tr><td style=\"width: 50%\">" +
												"<h1>Header1</h1>" +
												"<h2>Another Header2</h2>" +
												"<h3>A Header 3</h3>" +
												"<h3>A Header 4</h3>" +
												"<h3>A Header 3</h3>" +
												"<h2>Header2</h2>" +
												"</td>" +
												"<td style=\"vertical-align:top;\">" +
		                    "<div id=\"docout\"></div></td></tr></table></div>" );

			$fixture = $("<div id=\"fixture\"></div>");
      $("body").html( $content );
			$("body").append( $fixture );

			$bodyContent = $("#bodyContent");
		},

		afterEach: function() {
			$("body").html();
		}
	} );

	QUnit.test( "caches plugin instance", function( assert ) {
		$fixture.docout();

		assert.ok(
			$fixture.data( "plugin_docout" ),
			"has cached it into a jQuery data"
		);
	} );

	QUnit.test( "enable custom config", function( assert ) {
		$fixture.docout( {
			target: "foo-target",
			immediate: false,
			rootClass: "foo-root-class",
			childClass: "foo-child-class",
			elements: "foo-elements",
			replace: false,
			gntLink: "foo-generate-link",
			gntLinkWrap: "foo-link-container",
			gntRootWrap: "foo-root-outline-container",
			gntChildWrap: "foo-root-child-outline-container",
			gntId: "foo-generate-id"
		} );

		var pluginData = $fixture.data( "plugin_docout" );

		assert.deepEqual(
			pluginData.settings,
			{
			  "childClass": "foo-child-class",
			  "elements": "foo-elements",
			  "gntChildWrap": "foo-root-child-outline-container",
			  "gntId": "foo-generate-id",
			  "gntLink": "foo-generate-link",
			  "gntLinkWrap": "foo-link-container",
			  "gntRootWrap": "foo-root-outline-container",
			  "immediate": false,
			  "replace": false,
			  "rootClass": "foo-root-class",
			  "target": "foo-target"
			},
			"extend plugin settings"
		);

	} );

	QUnit.test( "document outline immediatelly", function( assert ) {
		$bodyContent.docout( { immediate: false });

		assert.equal(
			$(".docout-root-container").length,
			0,
			"is not run"
		);

		$bodyContent.docout("generate");

		assert.equal(
			$(".docout-root-wrap").length,
			1,
			"is explicitly executed"
		);

	} );

	QUnit.test( "renders the headings", function( assert ) {
		$bodyContent.docout( { target: "#docout" });

		assert.equal(
			$(".docout-root-wrap").length,
			1,
			"headers are ready"
		);

		assert.equal(
			$(".docout-root-wrap > li").length,
			1,
			"headers are ready"
		);

		assert.equal(
			$(".docout-root-wrap > li > ul.docout-child-wrap").length,
			1,
			"headers are ready"
		);

		assert.equal(
			$(".docout-root-wrap > li > ul.docout-child-wrap > li").length,
			2,
			"headers are ready"
		);

	} );
}( jQuery, QUnit ) );
